#!/bin/sh
# check for config file presence
if [ ! -f /etc/a5usb.rc ] ; then
  echo "ERROR 12: no configuration file /etc/a5usb.rc"
  exit 1
fi

# execute config file in the context of this script (take over config vars into local var space)
. /etc/a5usb.rc

# if no file name for the MBR file was specified...
if [ -z "${A5USB_IMAGE_MBR}" ]
then
  # try to derive the MBR file name from the size specification
  A5USB_IMAGE_MBR=/mnt_data/Leica3DD/bin/usbshare-${A5USB_SIZE}MB.mbr
fi

# if the MBR file is not existing...
if [ ! -e ${A5USB_IMAGE_MBR} ]
then
  echo "ERROR 13: bad config, MBR file missing (${A5USB_IMAGE_MBR})"
  exit 1
fi

# print determined setups for script operation
echo "A5USB_SIZE=${A5USB_SIZE}"
echo "A5USB_IMAGE=${A5USB_IMAGE}"
echo "A5USB_IMAGE_MBR=${A5USB_IMAGE_MBR}"
echo "A5USB_FSTYPE=${A5USB_FSTYPE}"
echo "A5USB_MOUNTPOINT=${A5USB_MOUNTPOINT}"

### helper functions ##
# prints a single word to the console telling if the USB image is currently mounted
isMounted()
  {
  if `grep $A5USB_MOUNTPOINT /proc/mounts > /dev/null >& /dev/null`; then
    echo -n "YES"
  else
    echo -n "NO"
  fi
  }

# prints a single word to the console telling if the USB image is currently exported via a USB endpoint
isExported()
  {
  if [ -e /sys/devices/platform/musb_hdrc/gadget/gadget-lun0/file ]; then
    echo -n "YES"
  else
    echo -n "NO"
  fi
  }

# detaches the USB image from the loop device if attached
loopDetach()
  {
  LOOP=`/sbin/losetup -a | grep $A5USB_IMAGE | cut -f 1 -d :`
  if [ ! -z "$LOOP" ]; then
    #echo "$LOOP end of loop"
    /sbin/losetup -d $LOOP
  fi
  }

# attaches the USB image to the next free loop device
loopAttach()
  {
    loopDetach # just make sure the image is not already hooked to any loop device

    for try in `seq 1 10`
    do
      LOOP_DEVICE=`losetup -f` # get full path of next free loop device
      losetup $LOOP_DEVICE $A5USB_IMAGE -o $((63 * 512))
                # attach USB image with offset to loopback device
                # (data partition starts at sector_id=63 with 512 bytes/sector, sector_id counting starts at sector_id=0)
      if [ $? -eq 0 ]; then break; fi
      LOOP_DEVICE=""
    done

    if [ -z "$LOOP_DEVICE" ]
    then
      echo "ERROR 18: failed attaching USB image to loop device"
      exit 1
    fi
  }

# unmounts and unloops the USB image from the system
unmount()
  {
  sync
  umount $A5USB_MOUNTPOINT >& /dev/null > /dev/null
  loopDetach
  sync
  }

# stops USB exporting of the system (removes and reloads needed kernel modules)
unexport()
  {
  sync
  /sbin/rmmod g_file_storage >& /dev/null > /dev/null
  /sbin/rmmod musb_hdr >& /dev/null > /dev/null
  # Put USB controller into host mode
  /sbin/modprobe musb_hdrc mode_default=1 >& /dev/null > /dev/null
  }

# stops the USB image export, unloops the USB image and then starts over mounting the image
doMount()
  {
  if [ ! -f $A5USB_IMAGE ]; then
      echo "ERROR 12: $A5USB_IMAGE does not exist: can not mount"
      exit 1
  fi

  if [ `isExported` == "YES" ] ; then
      unexport
  fi

  if [ `isExported` == "YES" ] ; then
      echo "ERROR 11: can't mount because share is exported"
      exit 1
  fi

  mkdir -p $A5USB_MOUNTPOINT # create mountpoint (if not already existing)

  loopAttach

  fsck.vfat -y $LOOP_DEVICE

  #echo mount $LOOP_DEVICE -o noatime $A5USB_MOUNTPOINT
  mount $LOOP_DEVICE -o noatime $A5USB_MOUNTPOINT > /dev/null
  if [ `isMounted` == "YES" ]; then
    echo "OK"
    mkdir -p $A5USB_MOUNTPOINT/export
    mkdir -p $A5USB_MOUNTPOINT/import
    mkdir -p $A5USB_MOUNTPOINT/update
    mkdir -p $A5USB_MOUNTPOINT/license
  else
    echo "ERROR 14: mounting USB image failed"
  fi
  }

doExport()
  {
  if [ `isExported` == "YES" ] ; then
    echo "OK" # Nothing to do here
    exit 1
  fi

  if [ `isMounted` == "YES" ] ; then
    unmount
  fi

  if [ `isMounted` == "YES" ] ; then
    echo "ERROR 11: can't export because share image is still mounted"
    exit 1
  fi

  loopAttach
  fsck.vfat -y $LOOP_DEVICE
  loopDetach

  /sbin/rmmod musb_hdrc 
  /sbin/modprobe musb_hdrc mode_default=2
  /sbin/modprobe g_file_storage file=$A5USB_IMAGE removable=Y

  if [ `isExported` == "YES" ] ; then
    echo "OK"
  else
    echo "ERROR 11: export failed"
  fi
  }

case "$1" in
  stop)
    unmount
    unexport
    if [ `isMounted` == "YES" ]; then
      echo "ERROR 11: USB share is still mounted"
    elif [ `isExported` == "YES" ]; then
      echo "ERROR 11: USB share is still exported"
    else
      echo "OK"
    fi
  ;;

  create)
    if [ -f $A5USB_IMAGE ]; then
      echo "ERROR 15: $A5USB_IMAGE already exists: will not overwrite"
      exit 1
    fi
    dd if=$A5USB_IMAGE_MBR of=$A5USB_IMAGE count=1 bs=512 # write MBR template to first sector of image
    if [ $? -ne 0 ]; then
      echo "ERROR 16: failed writing MBR to USB image"
      exit 1
    fi
    dd if=/dev/zero of=$A5USB_IMAGE count=$(($A5USB_SIZE * 1024 * 1024 / 512 - 1 )) bs=512 seek=1 # write zeros to rest of sectors in image
    if [ $? -ne 0 ]; then
      echo "ERROR 17: failed setting data area of USB image to zero"
      exit 1
    fi

    loopAttach

    mkfs.${A5USB_FSTYPE} -F 32 $LOOP_DEVICE
    if [ $? -ne 0 ]; then
      echo "ERROR 19: failed formatting data partition in USB image"
      exit 1
    fi

    # *** quote from wikipedia:
    # offset=0x03
    # length=8
    # OEM Name (padded with spaces 0x20).
    # This value determines in which system disk was formatted.
    # MS-DOS checks this field to determine which other parts of the boot record can be relied on.
    # Common examples are "IBM  3.3", "MSDOS5.0", "MSWIN4.1", "mkdosfs", and "FreeDOS".
    # *** quote from Microsoft: http://technet.microsoft.com/en-us/library/cc977221.aspx
    # Following the jump instruction is the 8-byte OEM ID,
    # a string of characters that identifies the name and version number of the operating system that formatted the volume.
    # To preserve compatibility with MS-DOS,
    # Windows 2000 records MSDOS5.0 in this field on FAT16 and FAT32 disks.
    # On NTFS disks, Windows 2000 records NTFS.
    # *** quote from the windows 7 console when checking a file system that has an OEM ID of "mkdosfs":
    # C:\Users\bog>chkdsk e:
    # The type of the file system is RAW.
    # CHKDSK is not available for RAW drives.
    echo "MSDOS5.0" >/mnt_data/OEM_ID.txt
    dd if=/mnt_data/OEM_ID.txt of=$LOOP_DEVICE bs=1 count=8 seek=3
    if [ $? -ne 0 ]; then
      echo "ERROR 19: failed patching OEM ID of data partition in USB image"
      exit 1
    fi
    # check if there is a backup boot sector given by its sector id
    dd if=$LOOP_DEVICE of=/mnt_data/BackupBootSectorId bs=1 count=2 skip=50
    a=`hexdump /mnt_data/BackupBootSectorId | cut -d ' ' -f 2 | head -n 1`; b=`echo 0x$a` ; c=`echo $(($b))`
    echo "BackupBootSectorId=$b ($c)"
    if [ $c -gt 0 ]
    then
      # read contents of backup boot sector from image
      dd if=$LOOP_DEVICE of=/mnt_data/BackupBootSector bs=512 count=1 skip=$c
      # determine byte location to modify
      seek=$(($c*512+3))
      # modify the location in question
      dd if=/mnt_data/OEM_ID.txt of=$LOOP_DEVICE bs=1 count=8 seek=$seek
    fi
  ;;
  
  restorefromimage)
    unmount
	unexport
	A5USB_IMAGE_ZIP=/mnt_data/Leica3DD/bin/usbshare.zip
	if [ -e "$A5USB_IMAGE_ZIP" ]; then
	  unzip -o "$A5USB_IMAGE_ZIP" -d /mnt_data/
	fi
  ;;
  
  delete)
    if [ `isMounted` == "YES" ]; then
      echo "ERROR 11: can't delete because USB share is mounted"
      exit 1
    fi

    if [ `isExported` == "YES" ]; then
      echo "ERROR 11: can't delete because USB share is exported"
      exit 1
    fi
    rm -f $A5USB_IMAGE
    echo "OK"
    ;;

  mount)
    doMount
  ;;

  export)
    doExport
  ;;

  status)
    if [ `isMounted` == "YES" ]; then
      echo "mounted"
    elif [ `isExported` == "YES" ]; then
      echo "exported"
    else
      echo "stopped"
    fi
  ;;

  *)
  echo "Usage: a5usb.sh mount|export|create|delete|stop"
  ;;
esac

exit 0
