#!/bin/sh -x
#
# Function:
#  - copy the specified file to the install directory
#  - unpack the file and run the install script
#  - changes the version information in versioninfo.xml
#  
#  The system is expected to be rebooted after running this script
#  to start the new version or rollback when the new SW fails

# the versioninfo file
versioninfo=versioninfo.xml

# the installer script
install=install_leica3dd.sh

# error codes
E_OPT=65
E_DEF=60
E_ROOT_DIR=3
E_TEMP_DIR=4
E_DIR=5
E_NOT_FOUND=6
E_PLATFORM=50
E_NEWVERSION=51

# utility functions
usage()
{
	echo "usage: `basename $0` -f <zip-file> [-c <curr-version>] [-n <new-version>]"
    echo ""
    echo "  -f <zip-file> : the compressed package to install"
    echo "  -c <curr-version> : the current version that runs, e.g. 0.3.4.1"
	echo "  -n <new-version> : the new version to run (default: taken from the zip file)"
	exit $E_OPT
}

osname=`uname -s`

# where to extract
if [ "$osname" == 'CYGWIN_NT-6.1' ]; then
	rootdir=/cygdrive/c/temp
elif [ "$osname" == Linux ]; then
	rootdir=/mnt_data
else
	echo "error: OS $osname not supported!"
	exit $E_PLATFORM
fi

# mandatory flags
fflag=
filename=
cvflag=
currversion=
nvflag=
newversion=

# parse command line
while getopts "f:c:n:" name
do
    case $name in
        f) fflag=1
			filename="$OPTARG";;
        c) cvflag=1
            currversion="$OPTARG";;
        n) nvflag=1
            newversion="$OPTARG";;
        ?) usage;;
    esac
done

# argument sanity checks
if [ -z "$filename" ]; then
	usage
elif [ ! -e "$filename" ]; then
	echo "error: $filename does not exist!"
    exit $E_NOT_FOUND
fi

if [ ! -e "$rootdir/$versioninfo" ]; then
	echo "error: $rootdir/$versioninfo does not exist!"
    exit $E_NOT_FOUND
fi

# take the version info from the specified filename if nothing is specified
installfilename=`basename $filename`
if [ -z "$newversion" ]; then
	newversion=`echo $installfilename | cut -d- -f2`
fi

# do a sanity check on the version info
grepout=`echo $newversion | grep '^[0-9][.][0-9][.][0-9][.][0-9]$'`

if [ -z "$grepout" ]; then
	echo "error: bad version $newversion specified!"
	exit $E_NEWVERSION
fi


# where the installer script is (relative to the $rootdir)
#bindir=`echo $installfilename | sed -e 's/release-install[.]zip//'`
installerdir=`echo $installfilename | cut -d- -f1,2,3`

# extract zip file to install directory
unzip -o -q $filename -d $rootdir

# run the install script - do *not* reboot (this is done by the starter)
/bin/sh -x $rootdir/$installerdir/$install -v $newversion -n || exit 1

# Change version information in versioninfo.xml
# This is done from the application.
# However, they are left here to test this script manually
if [ ! -z "$newversion" ] && [ ! -z "$currversion" ]; then
	sedscript=/tmp/cmds.sed
	echo "s/\(.*<Current>\).*\(<\/Current>.*\)/\1\2/1" > $sedscript
	echo "s/\(.*<Desired>\).*\(<\/Desired>.*\)/\1$newversion\2/1" >> $sedscript
	echo "s/\(.*<Rollback>\).*\(<\/Rollback>.*\)/\1$currversion\2/1" >> $sedscript

	sed -f $sedscript < $rootdir/$versioninfo > /tmp/$versioninfo.tmp
	mv /tmp/$versioninfo.tmp $rootdir/$versioninfo
	rm $sedscript
fi

exit 0
