#!/bin/bash

# file: installplatform.sh
#
# purpose:
# installs a new platform (=root file system image) into the system.
# further a file-flag is set that triggers a re-install of the application on next reboot.
# this script uses a symlink in order to make clear which system is the current system.

echo "installing platform '$1'..."


# variable setup
START_DIR=`pwd`

SRC_FILE=$1
SRC_NAME=`echo $SRC_FILE | sed -e 's/\(.*\/\)\(.*\)/\2/g'`
SRC_EXT=`echo $SRC_FILE | sed -n -e 's/^\(.*\.\)\(.*\)$/\2/p'`

if [ -z $SRC_FILE ]; then echo "missing parameter"; exit 1; fi
if [ -z $SRC_NAME ]; then echo "not a file name"; exit 1; fi
if [ -z $SRC_EXT ]; then echo "no extension"; exit 1; fi

DST_DIR=/mnt_data
DST_LINK=$DST_DIR/rootfs.img
DST_FILE_RELATIVE=$SRC_NAME
DST_FILE=$DST_DIR/$DST_FILE_RELATIVE

reinstall_flag=$DST_DIR/reinstall_flag


# if the current dir entry for the image is not a symlink then rename it and set a symlink on it
if [ ! -L $DST_LINK ]
then
    moved=100
    while [ $moved -ne 0 ]
    do
        if [ -e $DST_LINK.$moved ]
        then
                moved=$(($moved+1))
        else
                mv $DST_LINK $DST_LINK.$moved
                ln -s $DST_LINK.$moved $DST_LINK
                moved=0
        fi
    done
fi


# try to create the new file system image on target location - there are several cases...
image_on_target_location=0

if [ "$SRC_EXT" = "tar" ]
then
        cd $DST_DIR

        TAR_FILE=$SRC_FILE

        MD5_FILE=`tar -t -f $TAR_FILE | grep "\.md5$"`
        BZ2_FILE=`tar -t -f $TAR_FILE | grep "\.bz2$"`

        # check if needed files are contained in the 
        if [ -z $MD5_FILE ]; then echo "missing md5 file"; exit 1; fi
        if [ -z $BZ2_FILE ]; then echo "missing bz2 file"; exit 1; fi

        ROOTFS_IMG=`echo $BZ2_FILE | sed -e 's/\.bz2//g'`

        echo "untarring md5 info file..."
        tar -x -f $TAR_FILE $MD5_FILE

        echo "untarring root file system image..."
        tar -x -f $TAR_FILE $BZ2_FILE
        if [ $? -ne 0 ]; then echo "tar -x failed"; exit 1; fi

        echo "uncompressing root file system image..."
        bunzip2 -f $BZ2_FILE
        if [ $? -ne 0 ]; then echo "tar/bunzip2 failed"; exit 1; fi

        echo "verifying md5 sum of image..."
        # generate MD5 sum for the now unzipped root file system
        md5sum $ROOTFS_IMG >$ROOTFS_IMG.has.md5

        # get MD5 sum from reference listing
        cat $MD5_FILE | grep "  $ROOTFS_IMG$" >$ROOTFS_IMG.needs.md5

        # check MD5 sum for differences
        if [ `cat $ROOTFS_IMG.has.md5 | cut -d ' ' -f 1` != `cat $ROOTFS_IMG.needs.md5 | cut -d ' ' -f 1` ]; then echo "md5sum mismatch"; exit 1; fi

        DST_FILE_RELATIVE=$ROOTFS_IMG
        DST_FILE=`pwd`/$DST_FILE_RELATIVE
        image_on_target_location=1

        cd $START_DIR
fi

if [ "$SRC_EXT" = "bz2" ]
then
        cd $DST_DIR

        BZ2_FILE=$SRC_FILE

        ROOTFS_IMG=`echo $BZ2_FILE | sed -e 's/\.bz2//g'`

        echo "uncompressing root file system image..."
        bunzip2 -c -f $BZ2_FILE >$ROOTFS_IMG
        if [ $? -ne 0 ]; then echo "bunzip2 failed"; exit 1; fi

        DST_FILE_RELATIVE=$ROOTFS_IMG
        DST_FILE=`pwd`/$DST_FILE_RELATIVE
        image_on_target_location=1

        cd $START_DIR
fi

if [ $image_on_target_location -eq 0 ]
then
    if [ -e $DST_FILE ]
    then
        if [ $SRC_FILE = $DST_FILE ]
        then
                image_on_target_location=1
        else
                # get file inode numbers from both files and compare inode numbers
                inode1=`stat -c %i $SRC_FILE`
                inode2=`stat -c %i $DST_FILE`
                if [ $inode1 -eq $inode2 ]
                then
                        image_on_target_location=1
                fi
        fi
    fi

    if [ $image_on_target_location -eq 0 ]
    then
        # copy image file to destion dir
        cp -f $SRC_FILE $DST_FILE
   fi
fi


# change link onto destination file ("ln -s -f" seems to be buggy and does not work!)
rm -f $DST_LINK
ln -s -f $DST_FILE_RELATIVE $DST_LINK


# create file-flag that later on triggers application re-install
touch $reinstall_flag


echo "installing platform '$1' - done."
# ### EOF ###
