#!/bin/sh
#
# Enables this host's WLAN interface with the default or specified parameters
#
# Loads the drivers when required

exit_code=0

# our WLAN constants
WLAN_MODE=Ad-Hoc
# the sensor's WLAN interface address
WLAN_IFACE_SENSOR=192.168.87.81
# our WLAN device name
WLAN_DEV_NAME=tiwlan0

# optional parameters (with defaults)
eflag=
wlan_essid=3DD_00000000_Chn010
cflag=
wlan_channel=10
aflag=
wlan_iface=192.168.87.82

E_OPT=65

# utility functions
usage()
{
    echo "usage: `basename $0` [-e <essid>] [-c <channel>] [-a <ip-addr>]"
    echo ""
    echo "  -e <essid>   : the WLAN's desired ESSID (default: $wlan_essid)"
    echo "  -c <channel> : the desired channel to use {1...11} (default: $wlan_channel)"
    echo "  -a <ip-addr> : the desired IPv4 address for the WLAN interface (default: $wlan_iface)"
    exit $E_OPT
}

# parse command line
while getopts "e:c:a:" name
do
    case $name in
        e) eflag=1
            wlan_essid="$OPTARG";;
        c) cflag=1
            wlan_channel="$OPTARG";;
        a) aflag=1
            wlan_iface="$OPTARG";;
        ?) usage;;
    esac
done

# argument sanity checks

# check for top level kernel module
if [ `lsmod | grep -c "^tiwlan_drv"` -eq 0 ]
then
    # load kernel modules and WLAN adapter firmware
    cd /root/wifi || exit 4
    ./ti-load
    cd -
fi

# determine number of detected WLAN devices
WLAN_CARD_COUNT=`iwconfig 2>/dev/null | grep -c $WLAN_DEV_NAME`

if [ ${WLAN_CARD_COUNT} -eq 1 ]
then

    # determine the device name for the WLAN device
    wlan_dev=`iwconfig 2>/dev/null | grep $WLAN_DEV_NAME | cut -d" " -f 1`

	if [ "X$wlan_dev" == "X" ]; then
		echo "WLAN device $WLAN_DEV_NAME not found"
		exit 7
	fi

	up=0
	loop=1

	until [ $up -ge 1 ] || [ $loop -eq 0 ]
	do
		loop=`expr $loop - 1`

		retry=1
		wifi_up=`iwconfig $wlan_dev 2>/dev/null | grep -c $wlan_essid`
		iface_up=`ifconfig 2>/dev/null | grep -c $wlan_iface`
		# disable the ethernet interface when wifi is not up yet
		if [ $iface_up -gt 0 ] && [ $wifi_up -eq 0 ]; then
			ifconfig $wlan_dev down
		fi

		until [ "$wifi_up" -gt 0 ] || [ $retry -eq 0 ]
		do
			retry=`expr $retry - 1`
			# configure and activate HF part of WLAN device
			iwconfig $wlan_dev mode $WLAN_MODE || exit 1
			iwconfig $wlan_dev channel $wlan_channel || exit 2
			iwconfig $wlan_dev essid $wlan_essid || exit 3
			# let WLAN init complete - this should shrink the reported RTT value
			# of the first ping
			sleep 2

			wifi_up=`iwconfig $wlan_dev 2>/dev/null | grep -c $wlan_essid`
		done

		if [ $wifi_up -gt 0 ]; then
			echo "activated $wlan_dev in mode $WLAN_MODE with ESSID $wlan_essid channel $wlan_channel"
		else
			echo "failed to activate $wlan_dev in mode $WLAN_MODE with ESSID $wlan_essid channel $wlan_channel"
			#exit 8
		fi

		iface_up=`ifconfig 2>/dev/null | grep -c $wlan_iface`
		if [ $iface_up -lt 1 ]; then
			# assign ip address to WLAN device
			# (implicitly defined netmask is 255.255.255.0)
			ifconfig $wlan_dev $wlan_iface || exit 3
			# urgently needed - prior operations seemingly take
			# critical time until full start up
			sleep 2
			echo "assigned IP address $wlan_iface to $wlan_dev"
		fi

		# trigger connection startup by a small set of pings across the WLAN

		# try the measurement device
		ping -q -c 4 -W 1 $WLAN_IFACE_SENSOR
		if [ $? -ne 0 ]
		then
			ifconfig $wlan_dev down
			up=0
		else
			up=1
		fi
	done

	if [ $up -eq 1 ]; then
		echo "Sensor reached on $wlan_dev via $WLAN_IFACE_SENSOR      [DONE]"
	else
		echo "Sensor $WLAN_IFACE_SENSOR not reachable [FAILED]"
		exit_code=13
	fi
else
	echo "WLAN device $WLAN_DEV_NAME not found"
    exit_code=2
fi

exit $exit_code
