#!/bin/sh
#
# installation script that adapts the environment
# to run
#
# This script is run in ./leica3dd-x.x.x.x-mono

# where the versions are retained
versioninfo=versioninfo.xml
# the USB exported filesystem file
usbexport=usbshare.img

# error codes
E_OPT=65
E_DEF=60
E_ROOT_DIR=3
E_TEMP_DIR=4
E_DIR=5
E_NOT_FOUND=6
E_PLATFORM=50
E_NEWVERSION=51

# utility functions
usage()
{
	echo "usage: `basename $0` -v <version>"
    echo ""
    echo "  -v <version> : the version to set-up, e.g. 0.3.5.1"
	echo "  -n           : do not reboot when completed"
	exit $E_OPT
}

osname=`uname -s`

# where to extract
if [ "$osname" == 'CYGWIN_NT-6.1' ]; then
	installprefix=/cygdrive/c
	installdir=$installprefix/temp
elif [ "$osname" == Linux ]; then
	installprefix=
	installdir=/mnt_data
else
	echo "error: OS $osname not supported"
	exit $E_PLATFORM
fi

# mandatory flags
vflag=
version=
nflag=0

# parse command line
while getopts "v:n" name
do
    case $name in
        v) vflag=1
           version="$OPTARG";;
        n) nflag=1;;
        ?) usage;;
    esac
done

# argument sanity checks
if [ -z "$version" ]; then
	usage
fi

PROJ=Leica3DD
ARCH=leica3dd

dirname=$ARCH-$version-mono
root_dir=$installdir/$dirname
linkname=$installdir/$PROJ
bin_dir=$root_dir/bin
system_settings=Leica3DD.Core.Configuration.SystemSettings.xml
config_files="Leica3DD.Program.Main.exe.config Leica3DD.Program.Starter.exe.config"
start_script=$installdir/$PROJ/bin/start_$ARCH.sh
export_mount_pt=$installprefix/media/stick


echo "installing $version ..."


# substitute entries in SystemSettings to match
# the Archos platform
for x in `find $root_dir . -name $system_settings`
do
	sed -e 's/\(.*<IPAddrEther>\).*\(<\/IPAddrEther>.*\)/\1192.168.87.81\2/1' \
	    -e 's/\(.*<IPAddrRNDIS>\).*\(<\/IPAddrRNDIS>.*\)/\1192.168.86.81\2/1' \
		-e 's/\(.*<GetImageInterval>\).*\(<\/GetImageInterval>.*\)/\1200\2/1' \
		-e 's/\(.*<UdpTimeout>\).*\(<\/UdpTimeout>.*\)/\1500\2/1' \
		-e 's/\(.*<ScreenSnapRadius>\).*\(<\/ScreenSnapRadius>.*\)/\110\2/1' \
		-e 's/\(.*<FirstBoot>\).*\(<\/FirstBoot>.*\)/\1true\2/1' \
			< $x > $x.tmp
	mv $x.tmp $x
	echo -n "."
done

for y in $config_files
do
	for x in `find $root_dir . -name $y`
	do
		sed -e 's/INFO/WARN/g' \
		-e 's/DEBUG/WARN/g' \
		< $x > $x.tmp
		mv $x.tmp $x
	done
	echo -n "."
done

# set execute permissions on all scripts
chmod +x $root_dir/*.sh
chmod +x $bin_dir/*.sh

if [ -x "$bin_dir/aot.sh" ]; then
	echo "************** doing AOT compilation ***********"
	$bin_dir/aot.sh $bin_dir > $root_dir/aot.log 2>&1
	echo -n "."
else
	echo "warning: $bin_dir/aot.sh not executable or not found"
fi

# start Leica3DD from ICEwm startup
out=`grep $start_script /etc/icewm/startup`
if [ -z "$out" ]; then
	echo "TSLIB_TSDEVICE=/dev/input/touchscreen0 $start_script &" >> /etc/icewm/startup
	echo -n "."
fi

# mount point for USB stick
if [ ! -d $export_mount_pt ]; then
	mkdir $export_mount_pt
	echo -n "."
fi

# prevent that USB sticks are automounted when inserted
out=`grep /dev/sdb /etc/udev/mount.blacklist`
if [ -z "$out" ]; then
	echo "/dev/sdb?" >> /etc/udev/mount.blacklist
	echo -n "."
fi

# update kernel if platform has the scripts needed
if [ -e /root/mtd_init_read.sh ]; then
	if [ ! -e $installdir/mtd3.dump ]; then
		cp $bin_dir/mtd3.dump $installdir/mtd3.dump
		/root/mtd_init_write.sh $installdir/mtd3.dump
		nflag=0
	else
		cmp $bin_dir/mtd3.dump $installdir/mtd3.dump
		if [ $? -ne 0 ]; then
			rm -f $installdir/mtd3.dump
			cp $bin_dir/mtd3.dump $installdir/mtd3.dump
			/root/mtd_init_write.sh $installdir/mtd3.dump
			nflag=0
		fi
	fi
fi

# set-up an initial versioninfo.xml
if [ -e $installdir/$versioninfo ]; then
    file_is_good=1
    if [ `grep -c '<?xml version="1.0" encoding="utf-8"?>' $installdir/$versioninfo` -eq 0 ]; then file_is_good=0; fi
    if [ `grep -c '<Applications>'                         $installdir/$versioninfo` -eq 0 ]; then file_is_good=0; fi
    if [ `grep -c '</Applications>'                        $installdir/$versioninfo` -eq 0 ]; then file_is_good=0; fi
    if [ `grep -c '<Current>'                              $installdir/$versioninfo` -eq 0 ]; then file_is_good=0; fi
    if [ `grep -c '</Current>'                             $installdir/$versioninfo` -eq 0 ]; then file_is_good=0; fi
    if [ $file_is_good -eq 0 ]; then rm -f $installdir/$versioninfo; fi
fi
if [ ! -e $installdir/$versioninfo ]; then
	cp -p $bin_dir/$versioninfo.template $installdir/$versioninfo
fi

# create the USB exported filesystem file
if [ ! -e $installdir/$usbexport ]; then
	$bin_dir/a5usb_offset.sh create

	if [ ! -e $installdir/$usbexport ]; then
		echo "warning: failed to create  $installdir/$usbexport"
	fi
	echo -n "."
else
	if [ -e $installdir/$usbexport/update/.systemdiagnostics.log ]; then
		rm -f $installdir/$usbexport/update/.systemdiagnostics.log
	fi
fi


# create soft link to current installation
if [ -L "$linkname" ]; then
	rm -f $linkname
fi

if [ -d "$root_dir" ]; then
	ln -sf $root_dir $linkname
	echo -n "."
else
	echo "error: $root_dir not found"
	exit $E_NOT_FOUND
fi

# script that must exist
if [ ! -x $start_script ]; then
	echo "error: $start_script missing or not executable"
	exit $E_DIR
fi

echo " [DONE]"

# reboot, unless -n was specified
if [ $nflag -lt 1 ]; then
	#perform a file system check after reboot
	touch /mnt_data/forcefsck
	sync
	
	reboot
fi

exit 0
